"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.get_vault_growth = exports.get_vault_dfuse = exports.get_vault = void 0;
const dfuse_1 = require("./dfuse");
const utils_1 = require("./utils");
function get_vault(rpc, symcode) {
    return __awaiter(this, void 0, void 0, function* () {
        const code = "vaults.sx";
        const scope = code;
        const table = "vault";
        const results = yield rpc.get_table_rows({ json: true, code, scope, table, limit: 1, lower_bound: symcode, upper_bound: symcode });
        return results.rows[0];
    });
}
exports.get_vault = get_vault;
function get_vault_dfuse(client, symcode, block_num) {
    return __awaiter(this, void 0, void 0, function* () {
        return dfuse_1.stateTableRow(client, "vaults.sx", "vaults.sx", "vault", symcode, block_num);
    });
}
exports.get_vault_dfuse = get_vault_dfuse;
function get_vault_growth(client, symcode, block_num, block_num_delta = 172800) {
    return __awaiter(this, void 0, void 0, function* () {
        const block_num_previous = block_num - block_num_delta;
        const vault = yield get_vault_dfuse(client, symcode, block_num);
        const previous_vault = yield get_vault_dfuse(client, symcode, block_num_previous);
        // contract values
        const tvl = utils_1.toNumber(vault.deposit.quantity);
        const supply = utils_1.toNumber(vault.supply.quantity);
        const supply_previous = utils_1.toNumber(previous_vault.supply.quantity);
        const tvl_previous = utils_1.toNumber(previous_vault.deposit.quantity);
        const tvl_growth = tvl - tvl_previous;
        // value per share APY
        const virtual_price = supply / tvl;
        const virtual_price_previous = supply_previous / tvl_previous;
        const virtual_price_growth = (virtual_price_previous - virtual_price) / virtual_price; //** price growth reversed (previous - current)
        // calculate real growth APY
        const growth = tvl * virtual_price_growth; // approximate fees based on growth
        const tvl_average = (tvl_previous + tvl) / 2;
        const apy_average_revenue = growth * 365 / tvl_average;
        const apy_realtime_revenue = growth * 365 / tvl;
        return {
            // block information
            block_num,
            block_num_previous,
            block_num_delta,
            // contract values
            deposit: vault.deposit,
            staked: vault.staked,
            supply: vault.supply,
            account: vault.account,
            last_updated: vault.last_updated,
            // 24h computed values
            apy_average_revenue,
            apy_realtime_revenue,
            tvl,
            tvl_growth,
            growth,
            virtual_price,
            virtual_price_growth,
        };
    });
}
exports.get_vault_growth = get_vault_growth;
